/*
 *
 *   HElliZER: the first portable demo in the world
 *
 *   Copyright (C) 1996  Queue Members Group Art Division
 *   Coded by Mad Max / Queue Members Group (Mike Shirobokov)
 *   <mad_max@dixon.volgacom.samara.su>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 * 
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include "video.h"
#include "misc.h"
#include "music.h"

#include "SDL.h"

bool vidInitialized = false;
int frameCount = 0;

int vidSizeX, vidSizeY, vidPageSize, vidPageLength, vidBytesPerLine,
    vidMaxPage, vidVideoMode;

#define N_GAMMA 4
uchar cur_gamma=0, gamma[N_GAMMA][MAX_RGB+1];

SDL_Surface *screen;
Uint32 colors[256];
int pixel_size;
int start_time;

void vidInitVideo() {
  for( int i=0; i<N_GAMMA; i++ ) {
    for( int j=0; j<=MAX_RGB; j++ ) {
      int tmp = int((1+(double)i/N_GAMMA)*j);
      if( tmp>MAX_RGB ) tmp = MAX_RGB;
      gamma[i][j]=tmp;
    }
  }
  if ( SDL_Init(SDL_INIT_AUDIO|SDL_INIT_VIDEO) < 0 ) {
    error( "Cannot open display" );
  }
  vidInitialized = true;
}

void vidSetVideoMode() {
  int video_w, video_h;
  Uint32 video_flags;

  video_flags = (SDL_HWSURFACE|SDL_HWPALETTE|SDL_FULLSCREEN);
  video_w = (min( VID_MAX_SIZE_X, vidVideoMode ) & ~0x3);
fprintf(stderr, "Video width : %d\n", video_w);
  video_h = video_w*3/4;
  screen = SDL_SetVideoMode(video_w, video_h, 8, video_flags);
  if ( screen == NULL ) {
    error( "Cannot set video mode" );
  }
  vidBytesPerLine = vidSizeX = screen->w;
  vidSizeY = screen->h;
  vidPageSize = vidBytesPerLine*vidSizeY;
}

void vidCloseVideo() {
  if( vidInitialized ) {
    if ( frameCount ) {
      printf("Average frames per second = %5.2f\n",
	   frameCount*miscTimerRes/(miscTimer()-start_time) );
    }
    SDL_Quit();
  }
}

void vidChooseVideoMode() {
  if ( ! vidVideoMode )
    vidVideoMode = VID_MAX_SIZE_X/2;
  pixel_size=1;
}

void vidSaveConfig( FILE *h )
{
  fwrite( &vidVideoMode, sizeof(vidVideoMode), 1, h );
  fwrite( &pixel_size, sizeof(pixel_size), 1, h );
}

void vidLoadConfig( FILE *h )
{
  fread( &vidVideoMode, sizeof(vidVideoMode), 1, h );
  fread( &pixel_size, sizeof(pixel_size), 1, h );
}

int vidBlackColor;
RGB vidCurrentPalette[256];

void vidSetPalette( RGB* pal ) {
  memcpy( vidCurrentPalette, pal, sizeof(vidCurrentPalette) );
  if ( screen->format->palette ) {
    SDL_Color palent[256];

    for (int i=0; i<256; i++ ) {
      palent[i].r = gamma[cur_gamma][pal[i].red];
      palent[i].g = gamma[cur_gamma][pal[i].green];
      palent[i].b = gamma[cur_gamma][pal[i].blue];
    }
    SDL_SetColors(screen, palent, 0, 256);
  } else {
    for(int i=0; i<256; i++ ) {
      colors[i]=SDL_MapRGB(screen->format,
          gamma[cur_gamma][pal[i].red],
          gamma[cur_gamma][pal[i].green],
          gamma[cur_gamma][pal[i].blue]);
    }
  }
}

PAGE vidAllocPage( bool inRAM ) {
  return (PAGE)cmalloc(vidPageSize);
}

void vidFreePage( PAGE Page ) {
  cfree(Page);
}

void vidCopyPage( PAGE topage, PAGE frompage )
{
  memcpy( topage, frompage, vidPageSize );
}

void vidCopyPageT( PAGE topage, PAGE frompage )
{
  for( int i=0; i<vidPageSize; i++ )
    if( frompage[i] ) topage[i] = frompage[i];
}

void vidShowPage( PAGE Page )
{
  if( !start_time ) start_time = miscTimer();
  frameCount++;
  SDL_Event event;
  while( SDL_PollEvent(&event) ) {
    if ( event.type == SDL_QUIT) {
          Shutdown();
          exit(1);
    }
    if ( event.type == SDL_KEYDOWN ) {
      switch(event.key.keysym.sym) {
        case SDLK_F11: {
          cur_gamma = (cur_gamma+1)%N_GAMMA;
          vidSetPalette( vidCurrentPalette );
          break;
        }
        case SDLK_ESCAPE: {
          Shutdown();
          exit(1);
          break;
        }
      }
    }
  }
  if ( SDL_LockSurface(screen) < 0 ) {
    return;
  }
  switch (screen->format->BytesPerPixel) {
    case 1: {
        Uint8 *pixels;
        int i, x, y, s;

        s = screen->pitch - screen->w;
        pixels = (Uint8 *)screen->pixels;
        for ( i=0, y=screen->h; y; --y ) {
          for ( x=screen->w; x; --x )
            *pixels++ = Page[i++];
          pixels += s;
        }
      }
      break;
    case 2: {
        Uint16 *pixels;
        int i, x, y, s;

        s = screen->pitch/2 - screen->w;
        pixels = (Uint16 *)screen->pixels;
        for ( i=0, y=screen->h; y; --y ) {
          for ( x=screen->w; x; --x )
            *pixels++ = colors[Page[i++]];
          pixels += s;
        }
      }
      break;
    case 4: {
        Uint32 *pixels;
        int i, x, y, s;

        s = screen->pitch/4 - screen->w;
        pixels = (Uint32 *)screen->pixels;
        for ( i=0, y=screen->h; y; --y ) {
          for ( x=screen->w; x; --x )
            *pixels++ = colors[Page[i++]];
          pixels += s;
        }
      }
      break;
    default: {
      char str[256];
      SDL_UnlockSurface(screen);
      sprintf( str, "%d bits-per-pixel screen depth is not supported",
                          screen->format->BitsPerPixel );
      error(str);
    }
  }
  SDL_UnlockSurface(screen);
  SDL_UpdateRect(screen, 0, 0, 0, 0);
}

void vidClearPage( PAGE Page, int value ) {
  memset( Page, value, vidPageSize );
}

static int total;

void vidFlipPage( PAGE* vpage, PAGE vpage1, PAGE vpage2 )
{
  vidShowPage(*vpage);
  *vpage = PAGE( (int)vpage1 + (int)vpage2 - (int)*vpage );
}

void vidDitherPage( PAGE colorpage, PAGE bwpage, PAGE topage,
                    PAGE dithertable )
{
  for( int i=vidPageSize-1; i; i-- ) {
    topage[i] = dithertable[ (colorpage[i]<<8) + bwpage[i] ];
  }
}

void vidStartVideo()
{
}
